package yrpg.bamp.basedata.webapi;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import kd.bos.bd.log.enums.BDCtrlOperateSource;
import kd.bos.bd.log.helper.BDCtrlLogHelper;
import kd.bos.bd.service.AssignService;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.basedata.BaseDataResponse;
import kd.bos.exception.KDBizException;
import kd.bos.openapi.api.result.ApiSaveItemData;
import kd.bos.openapi.api.result.ApiSaveResult;
import kd.bos.openapi.api.result.ApiSaveRowErrorData;
import kd.bos.openapi.api.result.BaseFilterItemData;
import kd.bos.openapi.api.result.BaseFilterResult;
import kd.bos.openapi.common.custom.annotation.ApiController;
import kd.bos.openapi.common.custom.annotation.ApiParam;
import kd.bos.openapi.common.custom.annotation.ApiPostMapping;
import kd.bos.openapi.common.result.CustomApiResult;
import kd.bos.openapi.common.result.OpenApiResult;
import kd.bos.openapi.common.util.OpenApiSdkUtil;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.fi.dcm.common.util.StringUtils;
import yrpg.bamp.basedata.common.ApiReqModel;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;


@ApiController(value="CommonBill",desc="统一url入口")
public class SupplierApiController {
	//存放实际地址和接口入口的
	@ApiPostMapping(value="supplierURLApiHandler")
	public CustomApiResult<Object> supplierApiHandler(@Valid  @ApiParam("入口参数") ApiReqModel reqModel) {
		//获取入参
		String reqdata=reqModel.getData();
		JSONArray dataArray = JSONObject.parseArray(reqdata);
		if(dataArray.size()>500){
			return CustomApiResult.fail("-1","单次同步的供应商数据超过500条！");
		}
		// 新增（修改）、启用（解封）、禁用（封存） 入参
		JSONObject saveDataJson = new JSONObject();
		JSONObject freezeDataJson = new JSONObject();
		JSONObject unfreezeDataJson = new JSONObject();
		//解封时更新
		JSONObject unfreezeUpdateDataJson = new JSONObject();

		// 新增（修改）、启用（解封）、禁用（封存） data层参数
		JSONArray supplierData = new JSONArray();
		JSONArray freezeNums = new JSONArray();
		JSONArray unfreezeNums = new JSONArray();
		//解封时更新
		JSONArray unfreezeData = new JSONArray();

		JSONObject freezeNumsJson = new JSONObject();
		JSONObject unfreezeNumsJson = new JSONObject();
		Map jsonMap=new HashMap();
		List<String> mainids = new ArrayList<String>();
		for(int i=0;i<dataArray.size();i++) {
			JSONObject jsonObject = dataArray.getJSONObject(i);
			String number = jsonObject.getString("yrpg_mainid");
			JSONObject dataObject=dataArray.getJSONObject(i);
			JSONArray supplierOrg = dataObject.getJSONArray("supplier_org");
			jsonMap.put(number,supplierOrg);
			mainids.add(number);
		}
		QFilter filter = new QFilter("yrpg_mainid", QCP.in,mainids);
		//获取系统现有供应商数据
		DynamicObjectCollection suppliers = QueryServiceHelper.query("bd_supplier", "number,yrpg_mainid,enable", new QFilter[]{filter});
		Map<String,String> supplierStatusMap = new HashMap<String,String>();
		Map<String,String> supplierNumMap = new HashMap<String,String>();
		for (DynamicObject supplier : suppliers){
			supplierStatusMap.put(supplier.getString("yrpg_mainid"),supplier.getString("enable"));
			supplierNumMap.put(supplier.getString("number"),supplier.getString("yrpg_mainid"));
		}
		//新增废弃数据，执行跳过，接口返回参数
		JSONArray newfreezeData = new JSONArray();

		//循环判断部门进入保存、封存、解封集合
		for(int i=0;i<dataArray.size();i++){
			JSONObject jsonObject = dataArray.getJSONObject(i);
			String number = jsonObject.getString("yrpg_mainid");
			boolean isfreeze = jsonObject.getBoolean("isfreeze");
			//根据入参是否封存，已经系统中状态判断
			if(supplierStatusMap.get(number)==null&&isfreeze){
				newfreezeData.add(jsonObject);
				//禁用直接跳过
				continue;
			}
			//根据入参是否封存，已经系统中状态判断
			if(supplierStatusMap.get(number)==null){
				//新增
				supplierData.add(jsonObject);
			}else if(!isfreeze&&supplierStatusMap.get(number).equals("1")){
				//更新
				supplierData.add(jsonObject);
			}else if(supplierStatusMap.get(number).equals("1")&&isfreeze){
				//禁用
				freezeNums.add(number);
			}else if(supplierStatusMap.get(number).equals("0")&&!isfreeze){
				//启用更新
				unfreezeNums.add(number);
				unfreezeData.add(jsonObject);

			}
		}

		//返回值
		BaseFilterResult responseData = new BaseFilterResult();
		CustomApiResult finalResult =new CustomApiResult();
		//判断是否调用禁用启用
		boolean isfreeze = false;
		JSONObject resultJson = new JSONObject();
		List<JSONObject> resultList=new ArrayList<>();
		JSONObject jsonResult=null;
		int falseNum = 0;
		int successNum = 0;
		//调用禁用接口
		if(freezeNums.size()>0){
			isfreeze = true;
			freezeNumsJson.put("yrpg_mainids",freezeNums);
			freezeDataJson.put("data",freezeNumsJson);
			//调用接口
			OpenApiResult adminfreeze = OpenApiSdkUtil.invoke("/v2/yrpg/basedata/bd_supplier/supplierDisable", freezeDataJson, new HashMap<>());
			BaseFilterResult data = (BaseFilterResult)adminfreeze.getData();
			List<BaseFilterItemData> result = data.getResult();
			for(BaseFilterItemData item : result){
				jsonResult=new JSONObject();
				jsonResult.put("busiDataId",item.getId());
				jsonResult.put("mdmCode",item.getNumber());
				jsonResult.put("success",item.getErrors().size()==0?true:false);
				if(item.getErrors().size()!=0){
					String errorList = StringUtils.join(item.getErrors(),",");
					jsonResult.put("message",errorList);
					falseNum = falseNum + 1;
				}else{
					String successList = "消费该条数据成功！";
					jsonResult.put("message",successList);
					successNum = successNum + 1;
				}
				resultList.add(jsonResult);
			}
			//responseData = data;
		}
		//调用解封接口
		if(unfreezeNums.size()>0){
			unfreezeNumsJson.put("yrpg_mainids",unfreezeNums);
			unfreezeDataJson.put("data",unfreezeNumsJson);
			//调用接口
			OpenApiResult unfreezeResult = OpenApiSdkUtil.invoke("/v2/yrpg/basedata/bd_supplier/supplierEnable", unfreezeDataJson, new HashMap<>());
			unfreezeUpdateDataJson.put("data",unfreezeData);
			//调用接口
			OpenApiResult adminsave = OpenApiSdkUtil.invoke("/v2/yrpg/basedata/bd_supplier/suppliersave", unfreezeUpdateDataJson, new HashMap<>());
			ApiSaveResult data = (ApiSaveResult)adminsave.getData();
			//根据保存更新操作返回值结构，将保存返回值组装
			//AssembleResponseUtil.assembleResult(data,responseData,isfreeze);
			List<ApiSaveItemData> result = data.getResult();
			for(ApiSaveItemData item : result){
				jsonResult=new JSONObject();
				jsonResult.put("busiDataId",item.getId());
				jsonResult.put("mdmCode",item.getNumber());
				jsonResult.put("success",item.getErrors().size()==0?true:false);
				if(item.getErrors().size()!=0){
					String errorList = "";
					for(ApiSaveRowErrorData temp : item.getErrors()){
						errorList += StringUtils.join(temp.getRowMsg(),",");

					}
					jsonResult.put("message",errorList);
					falseNum = falseNum + 1;
				}else{
					String successList = "消费该条数据成功！";
					jsonResult.put("message",successList);
					successNum = successNum + 1;
				}
				resultList.add(jsonResult);
			}


		}
		//调用保存接口
		if(supplierData.size()>0){
			saveDataJson.put("data",supplierData);
			//调用接口
			OpenApiResult adminsave = OpenApiSdkUtil.invoke("/v2/yrpg/basedata/bd_supplier/suppliersave", saveDataJson, new HashMap<>());
			ApiSaveResult data = (ApiSaveResult)adminsave.getData();
			//根据保存更新操作返回值结构，将保存返回值组装
			List<ApiSaveItemData> result = data.getResult();
			for(ApiSaveItemData item : result){
				jsonResult=new JSONObject();
				jsonResult.put("busiDataId",item.getId());
				jsonResult.put("mdmCode",item.getKeys().get("yrpg_mainid"));
				jsonResult.put("success",item.getErrors().size()==0?true:false);
				if(item.getErrors().size()!=0){
					String errorList = "";
					for(ApiSaveRowErrorData temp : item.getErrors()){
						errorList += StringUtils.join(temp.getRowMsg(),",");

					}
					jsonResult.put("message",errorList);
					falseNum = falseNum + 1;
				}else{
					String successList = "消费该条数据成功！";
					jsonResult.put("message",successList);
					//调用分配策略
					List<Long> dataIds = new ArrayList();
					dataIds.add(Long.parseLong(item.getId()));
					Set<Long> orgIds = new HashSet();
					List<String> codes=new ArrayList<>();

					JSONArray supplierOrg = (JSONArray) jsonMap.get(item.getKeys().get("yrpg_mainid"));
					for(int n=0;n<supplierOrg.size();n++){
						codes.add(supplierOrg.getJSONObject(n).getString("code"));
					}
					if(codes.size()>0){
						QFilter filtersup = new QFilter("yrpg_companynum", QCP.in,codes);//单据状态
						QFilter[] supfilters = new QFilter[]{filtersup};
						DynamicObject[] orgbills= BusinessDataServiceHelper.load("bos_adminorg","id",supfilters);
						for(DynamicObject orgbill:orgbills){
							orgIds.add(Long.parseLong(orgbill.getString("id")));
						}

						if(orgIds.size()>0){
							String entity = "bd_supplier";
							Long useOrgId = 100000L;
							List<Long> dataIdsTemp = new ArrayList(dataIds.size());
							dataIdsTemp.addAll(dataIds);
							String appId = "basedata";
							AssignService assignService = new AssignService(entity, false);
							BDCtrlLogHelper.setLogSourceType(BDCtrlOperateSource.INTERFACE_OP);
							BaseDataResponse response = assignService.assign(useOrgId, appId, new HashSet(dataIdsTemp), orgIds, false);
							if (response.isSuccess() == false) {
								String assignList = "该条分配失败！";
								jsonResult.put("message",assignList);
								falseNum++;
							}else{
								successNum++;
							}
						}else{
							successNum++;
						}
					}else{
						successNum++;
					}

				}
				resultList.add(jsonResult);

			}
		}
		//多类型接口 过滤默认为空
		resultJson.put("mdMappings",resultList);
		resultJson.put("success",true);
		resultJson.put("message","本次主表装载成功" + successNum + " 条,失败 "+ falseNum + " 条");
		return CustomApiResult.success(resultJson);
	}

}

